/*******************************************************
 ***
 ***      Arithmetic Interpreter        Version 2.2
 ***
 ***                      C Version
 ***
 *******************************************************/

/*

Cette portion de code permet de reconnaitre des expressions mathematiques
rentrees sous forme de chaines de caracteres et ensuite d'evaluer ces ex-
pressions.

Les fonctions reconnues sont:
+ - * / ^
exp ln log abs sin cos tan asin acos atan sinh (ou sh) cosh (ou ch)
tanh (ou th)
L'interpreteur reconnait les nombres reels,entiers positifs ou negatifs.
L'interpreteur reconnait les variables x et y.

L'interpreteur sait evaluer des expressions avec redondance de parentheses.
Exemple: (1+(2)) donnera 3.

Les fonctions unaires (sin,cos,abs,....), si elles n'admettent qu'un seul 
argument (Exemple sin(x)) peuvent etre ecrite aussi sans parentheses (Exemple:
sinx).   ATTENTION: ecrire sinx+3 est equivalent a sin(x)+3.

Pour beneficier de cet interpreteur, il suffit d'inclure le fichier interprete.c
dans le projet et de faire appel aux fonctions:

void ConstructTree(CNode *,char *,int,short *);

et

float ComputeTree(CNode *,float,float,short *);

La fonction ConstructTree sert a construire l'arbre binaire representant l'expres-
sion saisie au clavier.Elle prend en arguments l'addresse d'une structure CNode,
une chaine de caractere a interpreter, le nombre de caracteres de cette
chaine (signe '\0' non inclus) et un short qui representera le type d'erreur rencontree.

IMPORTANT: les lettres doivent etre en minuscule et il ne doit pas y'avoir d'espace
sous peine de renvoi d'un code d'erreur 1,2 ou 3.

exemple: 3*x+1 est correct. 3*X+1  non      -> erreur de type 3. 
                            3* x+1 non plus -> erreur de type 3.

La fonction ComputeTree prend en arguments l'addresse d'une structure CNode a e-
valuer, les valeurs respectives des variables x et y et un short qui representera
le type d'erreur rencontree.


Statistiques:
	
Sur la fonction: 1/(x+0.2)*sin(x^2)*cos(x), on obtient ce resultat somme toute correct:
				
 86500 evaluations/seconde sur un PowerMac 7200/90
266000 evaluations/seconde sur une SUN Ultra 5 Solaris.
				
Teste sur:

SUN Ultra 5:
SOLARIS                 g++ interpreteur.c -lm

PowerMac G3:
MacOS X Server (RDR2)
MacOS 8.x               CodeWarrior IDE

Historique:

2.2:
    * Version Objective-C disponible.
    * Amelioration du code de la version C++
      - Correction de bugs sur l'operator =
      - Optimisation du code
      - Reformatage pour plus de clarete.
    * Mise a jour du code de la version C.

2.1:
  * Version Java disponible:
    # les operateurs mathematiques disponibles sont ceux de Java donc certains manquent.

  * Correction d'un bug sur le calcul de puissance (Version C++).

2.0: 
  * Portage en C++ et debut en Java

Version C++:
	 
  * Meilleure gestion memoire lors de la construction de l'expression.
  * Acceleration de certaines operations.

Version Java:
	 
  * Premiere version. Alors si il y'a un bug, c'est fort possible.

1.3b: ajoute les fonctions suivantes: (NON DISTRIBUEE)
  * reconnaissance du symbole 
  * ecriture formatee d'expression (a ameliorer)

1.2: corrige les bugs suivants:
    * erreur sur l'interpretation de fonctions unaires imbriquees telles que
      ln(exp(x)).
    * la fonction puissance (pour les puissances entieres).
  ajoute:
    * la verification de la chaine entree (voir code d'erreur)
    * la verification de l'existence de l'evaluation (voir code d'erreur)
     

1.1: corrige un bug au niveau de l'interpretation des fonctions du type:

     exp(-(x+y))

1.0: premiere version


Ce code peut etre distribue et modifie librement.

*/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "interpreteur.h"

float PuisInt(float x,short y) {
short i;
float res=1;

for(i=0;i<y;i++) res*=x;
return res;
}

char * CopyPartialString(char * inString,short inBegin,short inEnd) {
char *  tString;
short tLength=inEnd-inBegin+1;

tString=(char *) calloc(tLength+1,sizeof(char));

if (tString!=NULL) {
  memmove(tString,inString+inBegin,tLength);
  tString[tLength]='\0';
}

return tString;
}

float CalcValue(char * inString,short inIndex) {
short i=inIndex,j;
float tValue=0;
short sign=1;

if (inString[inIndex]=='') return PI;
if (inString[inIndex]=='-') {
	sign=-1;
	i++;
}

while (inString[i]>47 && inString[i]<58) {
	tValue=tValue*10+(inString[i]-'0');
	i++;
}

if (inString[i]=='.') {
	i++;
	j=1;
	
  while (inString[i]>47 && inString[i]<58) {
		tValue=tValue+(inString[i]-'0')*exp(-j*2.30258509);
		i++;
		j++;
	}
}

return (sign*tValue);
}

/* recherche de l'operateur mathematique */

void FindOperator(short * oper,short * esp,char * inString,short pos) {

switch(inString[pos]) {
	case 'a':
		switch(inString[pos+1]) {
			case 'b':
				*esp=3;
				*oper=10;
				break;
			case 'c':
				*esp=4;
				*oper=15;
				break;
			case 's':
				*esp=4;
				*oper=14;
				break;
			case 't':
				*esp=4;
				*oper=16;
				break;
		}
		break;
	case 'c':
		if (inString[pos+1]=='h') {
			*esp=2;
			*oper=18;
		}
		else if (inString[pos+1]=='o' && inString[pos+2]=='s') {
			if (inString[pos+3]=='h') {
				*esp=4;
				*oper=18;
			}
			else {
        *esp=3;
				*oper=12;
			}
		}
		break;
	case 'e':
		if (inString[pos+1]=='x' && inString[pos+2]=='p') {
			*esp=3;
      *oper=6;
		}
		else *oper=-10;
		break;
	case 'l':
		if (inString[pos+1]=='n') {
			*esp=2;
			*oper=7;
		}
		else if (inString[pos+1]=='o' && inString[pos+2]=='g'){
         	*esp=3;
          *oper=8;
         }
         else *oper=-10;
         break;
	case 's':
		if (inString[pos+1]=='h') {
			*esp=2;
			*oper=17;
		}
		else if (inString[pos+1]=='q') {
			*esp=4;
			*oper=9;
		}
		else {
			if (inString[pos+3]=='h') {
				*esp=4;
				*oper=17;
			}
			else {
				*esp=3;
				*oper=11;
			}
		}
		break;
	case 't':
		if (inString[pos+1]=='h') {
			*esp=2;
			*oper=19;
		}
		else if (inString[pos+1]=='a' && inString[pos+2]=='n') {
			if (inString[pos+3]=='h') {
				*esp=4;
				*oper=19;
			}
			else {
				*esp=3;
				*oper=13;
			}
		}
		else *oper=-10;
		break;
	default:
		*oper=-10;
		break;
}
}

/* manipulation des arbres */

CNode * NewNode(void) {
CNode * tNode;

tNode=(CNode *) calloc(1,sizeof(CNode));
tNode->fg=NULL;
tNode->fd=NULL;

return tNode;
}

/* Destruction d'un arbre */

void DeleteNode(CNode * inNode) {

if (inNode!=NULL) {
	DeleteNode(inNode->fg);
	DeleteNode(inNode->fd);
	free(inNode);
}
}

/* Duplication d'un arbre */

CNode * CopyNode(CNode * inNode) {
CNode * tNode=NULL;

if (inNode!=NULL) {
  tNode=NewNode();

  tNode->operator_=inNode->operator_;
  tNode->val=inNode->val;

  tNode->fg=CopyNode(inNode->fg);
  tNode->fd=CopyNode(inNode->fd);
}

return tNode;
}



/*********************************************
 *      Definition des codes d'erreurs       *
 *                                           *
 *    0 : pas d'erreur                       *
 *    1 : erreur de parenthese               *
 *    2 : symbole non reconnu                *
 *    3 : argument absent                    *
 *                                           *
 *********************************************/

void ConstructTree(CNode * newn,char * chaine,short longueur,short * err) {
short imbric,Bimbric;
short priorite,ope;
short position,positionv,i,j;
short opetemp=0;
short espa,espat;
short caspp=0;

/* Initialisation des variables */

if (longueur<=0) {
	*err=3;
	return;
}

ope=0;
imbric=0;Bimbric=128;
priorite=6;
i=0;
positionv=position=0;

/* Mise en place des donnees sur le morceau de chaine */

while(i<longueur) {
	/*printf("%c",chaine[i]);*/
	if ((chaine[i]>47 && chaine[i]<58) || (chaine[i]=='')) {
		if (priorite>5) {
			priorite=5;
			positionv=i;
		}
		i++;
	}
	else {
	if (chaine[i]>96 && chaine[i]<117) {
		FindOperator(&opetemp,&espat,chaine,i);
		if (opetemp>=0) {
			if (imbric<Bimbric) {
				Bimbric=imbric;
				ope=opetemp;
				position=i;
				priorite=4;
				espa=espat;
			} 
			else if (imbric==Bimbric && priorite >=4) {
				ope=opetemp;
				position=i;
				priorite=4;
				espa=espat;
			}
			j=i+1;
			i+=espat;
			while(j<i) {
				/*printf("%c",chaine[j]);*/
				j++;
			}
		}
		else if (chaine[i]=='t') {
			if (priorite==6) ope=-1;
			i++;
		}
		else if (chaine[i]=='p') {
			if (priorite==6) ope=-2;
			i++;
		}
		else if (chaine[i]=='r') {
			if (priorite==6) ope=-2;
			i++;
		}
		else if (chaine[i]=='n') {
			if (priorite==6) ope=-1;
			i++;
		}
		else {  /* symbole non reconnu */
			*err=2; 
      return;
    }
	}
	else {
		switch(chaine[i]) {
			case '(':
				imbric++;
				break;
			case ')':
				imbric--;
				break;
			case '+':
				if (imbric<Bimbric) {
					Bimbric=imbric;
					priorite=1;
					ope=1;
					position=i;
					caspp=0;
				}
				else if (imbric==Bimbric && priorite >=1) {
					priorite=1;
					ope=1;
					position=i;
					caspp=0;
				}
				break;
			case '-':
				if (imbric<Bimbric) {
					if ((i-1)<0) {
						if (priorite>5) {
							priorite=1;
							position=i;
							ope=2;
							Bimbric=imbric;
							caspp=1;
						}
					}
				 	else if (chaine[i-1]=='(') {
				 		if (priorite>1){
				 			priorite=1;
				 			position=i;
				 			Bimbric=imbric;
				 			caspp=1;
				 			ope=2;
				 		}
				 	}
					else {
						Bimbric=imbric;
						priorite=1;
						ope=2;
						position=i;
						caspp=0;
					}
				}
				else if (imbric==Bimbric && priorite>=1) {
					if ((i-1)<0) {
						if (priorite>5) {
							priorite=1;
							position=i;
							ope=2;
							caspp=1;
						}
					}
					else if (chaine[i-1]=='(') {
						if (priorite>1){
							priorite=1;
							position=i;
							caspp=1;
							ope=2;
						}
					}
					else {
						priorite=1;
						ope=2;
						position=i;
						caspp=0;
					}
				}
				break;
			case '*':
				if (imbric<Bimbric) {
					Bimbric=imbric;
					priorite=2;
					ope=3;
					position=i;
				}
				else if (imbric==Bimbric && priorite>=2) {
					priorite=2;
					ope=3;
					position=i;
				}
				break;
			case '/':
				if (imbric<Bimbric) {
					Bimbric=imbric;
					priorite=2;
					ope=4;
					position=i;
				}
				else if (imbric==Bimbric && priorite>=2) {
					priorite=2;
					ope=4;
					position=i;
				}
				break;
			case '^':
				if (imbric<Bimbric) {
					Bimbric=imbric;
					priorite=3;
					ope=5;
					position=i;
				}
				else if (imbric==Bimbric && priorite>=3) {
					priorite=3;
					ope=5;
					position=i;
				}
				break;
			case '.':
				break;
			case 'x':
				if (priorite==6) ope=-1;
				break;
			case 'y':
				if (priorite==6) ope=-2;
				break;
			default:
				*err=2; /* symbole non reconnu */
				return;
			}
      i++;
		}
	}
}

if (imbric!=0) { /* erreur de parenthesage */
	*err=1;
	return;
}

/* Traitement des donnees	*/

/*printf("\nMeilleure Imbrication: %d Meilleure priorite: %d Operateur: %d",Bimbric,priorite,ope);
printf(" Position: %d Positionv: %d\n",position,positionv);
printf("caspp: %d\n",caspp);*/


if (priorite==6) {
  newn->operator_=ope;
}
else if (caspp==1) {
  newn->operator_=2;
	newn->fg=NewNode();
	newn->fd=NewNode();
  
	newn->fg->operator_=0;
	newn->fg->val=0;
  
	if ((longueur-position-1-Bimbric)==0) { /* Argument absent */
    *err=3;
    return;
  }
	
  ConstructTree(newn->fd,CopyPartialString(chaine,(position+1),(longueur-1-Bimbric)),(longueur-position-1-Bimbric),err);
}

else if (priorite==5) {
  newn->operator_=0;
  newn->val=CalcValue(chaine,positionv);
}
else if (ope>5) {
  newn->operator_=ope;
  newn->fg=NewNode();
  
  if ((longueur-position-espa-Bimbric)==0) { /* Argument absent */
    *err=3;
    return;
  }
  ConstructTree(newn->fg,CopyPartialString(chaine,(position+espa),(longueur-1)),(longueur-position-espa-Bimbric),err);
				  } 
else {
  newn->operator_=ope;
  newn->fg=NewNode();
  newn->fd=NewNode();
  
  if ((position-Bimbric)==0) { /* Argument absent */
    *err=3;
    return;
  }
  
  ConstructTree(newn->fg,CopyPartialString(chaine,Bimbric,(position-1)),(position-Bimbric),err);
  
  if ((longueur-position-1-Bimbric)==0) { /* Argument absent */
    *err=3;
    return;
  }
  
  ConstructTree(newn->fd,CopyPartialString(chaine,(position+1),(longueur-1-Bimbric)),(longueur-position-1-Bimbric),err);
}
}



/*********************************************
 *      Definition des codes d'erreurs       *
 *                                           *
 *    0 : pas d'erreur                       *
 *    1 : resultat infini                    *
 *    2 : non defini                         *
 *                                           *
 *********************************************/

CNode * Deriv_fpg_arbre(CNode * theTree) {
CNode * newTree;

newTree=NewNode();
newTree->operator_=3;
newTree->fg=Deriv_arbre(theTree->fg);
newTree->fd=CopyNode(theTree->fd);
return newTree;
}

CNode * Deriv_fgp_arbre(CNode * theTree) {
CNode * newTree;

newTree=NewNode();
newTree->operator_=3;
newTree->fg=CopyNode(theTree->fg);
newTree->fd=Deriv_arbre(theTree->fd);
return newTree;
}

CNode * Deriv_frac_num(CNode * theTree) {
CNode * newTree;

newTree=NewNode();
newTree->operator_=2;
newTree->fg=Deriv_fpg_arbre(theTree);
newTree->fd=Deriv_fgp_arbre(theTree);
return newTree;
}

CNode * Deriv_frac_den(CNode * theTree) {
CNode * newTree;

newTree=NewNode();
newTree->operator_=5;
newTree->fg=CopyNode(theTree->fd);
newTree->fd=NewNode();
newTree->fd->operator_=0;
newTree->fd->val=2;

return newTree;
}

CNode * Deriv_arbre(CNode * theTree) {
CNode * newTree;

newTree=NewNode();

if (theTree->operator_==0) {
	newTree->operator_=0;
	newTree->val=0;
	return newTree;
}
if (theTree->operator_==-1) {
	newTree->operator_=0;
	newTree->val=1;
	return newTree;
}
if (theTree->operator_==-2) {
	newTree->operator_=0;
	newTree->val=1;
	return newTree;
}
else {
	switch(theTree->operator_) {
		case 1:
			newTree->operator_=1;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=Deriv_arbre(theTree->fd);
			return newTree;
			break;
		case 2:
			newTree->operator_=2;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=Deriv_arbre(theTree->fd);
			return newTree;
			break;
		case 3:
			newTree->operator_=1;
			newTree->fg=Deriv_fpg_arbre(theTree);
			newTree->fd=Deriv_fgp_arbre(theTree);
			return newTree;
			break;
		case 4:
			newTree->operator_=4;
			newTree->fg=Deriv_frac_num(theTree);
			newTree->fd=Deriv_frac_den(theTree);
			return newTree;
			break;
		case 5:
			newTree->operator_=3;
			newTree->fg=NewNode();
			newTree->fg->operator_=3;
			newTree->fg->fg=CopyNode(theTree->fd);
			newTree->fg->fd=Deriv_arbre(theTree->fg);
			newTree->fd=NewNode();
			newTree->fd->operator_=5;
			newTree->fd->fg=CopyNode(theTree->fg);
			newTree->fd->fd=NewNode();
			newTree->fd->fd->operator_=0;
			newTree->fd->fd->val=theTree->fd->val-1.0;
			
			return newTree;
			break;
		case 6: /* exponentielle */
			newTree->operator_=3;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=CopyNode(theTree);
			return newTree;
			break;
		case 7: /* logarithme neperien */
			newTree->operator_=4;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=CopyNode(theTree->fg);
			return newTree;
			break;
		case 8: /* logarithme decimal */
			newTree->operator_=3;
			newTree->fg=NewNode();
			newTree->fg->operator_=7;
			newTree->fg->fg=NewNode();
			newTree->fg->fg->operator_=0;
			newTree->fg->fg->val=0.1;
			
			newTree->fd=NewNode();
			newTree->fd->operator_=4;
			newTree->fd->fg=Deriv_arbre(theTree->fg);
			newTree->fd->fd=CopyNode(theTree->fg);
			return newTree;
			break;
		case 9: /* racine carree */
			newTree->operator_=4;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=NewNode();
			newTree->fd->operator_=3;
			newTree->fd->fg=NewNode();
			newTree->fd->fg->operator_=0;
			newTree->fd->fg->val=2;
			newTree->fd->fg->fg=newTree->fd->fg->fd=NULL;
			newTree->fd->fd=CopyNode(theTree->fg);
			return newTree;
			break;
		case 10: /* Valeur absolue a revoir */
			newTree=Deriv_arbre(theTree->fg);
			return newTree;
			break;
		case 11: /* sinus */
			newTree->operator_=3;
			newTree->fg=Deriv_arbre(theTree->fg);
			newTree->fd=NewNode();
			newTree->fd->operator_=12;
			newTree->fd->fg=CopyNode(theTree->fg);
			
			return newTree;
			break;
		case 12: /* cosinus */
			newTree->operator_=3;
			newTree->fg=NewNode();
			newTree->fd=NewNode();
			newTree->fg->operator_=2;
			newTree->fg->fg=NewNode();
			newTree->fg->fg->operator_=0;
			newTree->fg->fg->val=0;
			newTree->fg->fg->fg=newTree->fg->fg->fd=NULL;
			newTree->fg->fd=Deriv_arbre(theTree->fg);
			newTree->fd->operator_=11;
			newTree->fd->fg=CopyNode(theTree->fg);
			
			return newTree;
			break;
		case 13: /* tangente */
			break;
	
	}
}
return NULL;
}
	
float ComputeTree(CNode * inNode,float valx,float valy,short * Err) {
register float valueL,valueR;
register short tOperator=inNode->operator_;

if (tOperator==0) return inNode->val;
if (tOperator==-1) return valx;
if (tOperator==-2) return valy;

valueL=ComputeTree(inNode->fg,valx,valy,Err);

if (*Err!=0) return 0;

if (inNode->fd!=NULL) valueR=ComputeTree(inNode->fd,valx,valy,Err);
if (*Err!=0) return 0;

switch (tOperator) {
	case 1:
		return (valueL+valueR);

	case 2:
		return (valueL-valueR);

	case 3:
		return (valueL*valueR);

	case 4:
		if (valueR==0) {
			*Err=1;
			return 0;
		}
		else  return (valueL*1.0/valueR);

	case 5:
		if ((((int) valueR)-valueR)==0) {
			return pow(valueL,valueR);

		}
		else if (valueL<0) {
			*Err=2;
			return 0;
		} 
		else {
			return pow(valueL,valueR);

		}

	case 6:
		return exp(valueL);

	case 7:
		if (valueL<=0) {
			if (valueL<0) *Err=2;
			else *Err=1;
			return 0;
		}
		else  return log(valueL);

	case 8:
		if (valueL<=0) {
			if (valueL<0) *Err=2;
			else *Err=1;
			return 0;
		}
		else  return log10(valueL);

	case 9:
		if (valueL<0) {
			*Err=2;
			return 0;
		}
		else  return sqrt(valueL);

	case 10:
		return fabs(valueL);

	case 11:
		return sin(valueL);

	case 12:
		return cos(valueL);

	case 13:
		return tan(valueL);

	case 14:
		return asin(valueL);

	case 15:
		return acos(valueL);

	case 16:
		return atan(valueL);

	case 17:
		return sinh(valueL);

	case 18:
		return cosh(valueL);

	case 19:
		return tanh(valueL);

}

return 0;
}
   
void WriteTree(CNode * inNode,char * chaine) {
int parenthese=0;
char temp[20];

if (inNode==NULL) return;
switch(inNode->operator_) {
	case -2:
		strcat((char *) chaine,"y");
		break;
	case -1:
		strcat((char *) chaine,"x");
		break;
	case 0:
		
		sprintf((char *) temp,"%f",inNode->val);
		strcat((char *) chaine,(char *) temp);
		break;
	case 1:
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,"+");
		WriteTree(inNode->fd,chaine);
		break;
	case 2:
		if (inNode->fg->operator_!=0 && inNode->fg->val!=0)
		  WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,"-");
		if (inNode->fd->operator_==1 || inNode->fd->operator_==2) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fd,chaine);
		if (parenthese==1) {
			strcat((char *) chaine,")");
		}
		break;
	case 3:
		if (inNode->fg->operator_==1 || inNode->fg->operator_==2) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fg,chaine);
		if (parenthese==1)
			strcat((char *) chaine,")");
		parenthese=0;
		strcat((char *) chaine,"*");
		if (inNode->fd->operator_==1 || inNode->fd->operator_==2) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fd,chaine);
		if (parenthese==1)
			strcat((char *) chaine,")");
		break;
	case 4:
		if (inNode->fg->operator_==1 || inNode->fg->operator_==2) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fg,chaine);
		if (parenthese==1)
			strcat((char *) chaine,")");
		parenthese=0;
		strcat((char *) chaine,"/");
		if (inNode->fd->operator_>0 && inNode->fd->operator_<5) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fd,chaine);
		if (parenthese==1) 
			strcat((char *) chaine,")");
		break;
	case 5:
		if (inNode->fg->operator_>0 && inNode->fg->operator_<5) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fg,chaine);
		if (parenthese==1)
			strcat((char *) chaine,")");
		parenthese=0;
		strcat((char *) chaine,"^");
		if (inNode->fd->operator_>0 && inNode->fd->operator_<5) {
			parenthese=1;
			strcat((char *) chaine,"(");
		}
		WriteTree(inNode->fd,chaine);
		if (parenthese==1) 
			strcat((char *) chaine,")");
		break;
	case 6:
		strcat((char *) chaine,"exp(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 7:
		strcat((char *) chaine,"log(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 8:
		strcat((char *) chaine,"ln(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 9:
		strcat((char *) chaine,"sqrt(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 10:
		strcat((char *) chaine,"|");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,"|");
		break;
	case 11:
		strcat((char *) chaine,"sin(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 12:
		strcat((char *) chaine,"cos(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 13:
		strcat((char *) chaine,"tan(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 14:
		strcat((char *) chaine,"asin(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 15:
		strcat((char *) chaine,"acos(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 16:
		strcat((char *) chaine,"atan(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 17:
		strcat((char *) chaine,"sinh(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 18:
		strcat((char *) chaine,"cosh(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
	case 19:
		strcat((char *) chaine,"tanh(");
		WriteTree(inNode->fg,chaine);
		strcat((char *) chaine,")");
		break;
}
		
}


/*************************************************************************/

int main(int argc, char ** argv) {
char tString2[500];
char tString[256];
CNode * test;
float resu;
short  err=0;
long pos=0;

test=NewNode();
printf("Entrez votre expression parenthesee: \n");
scanf("%s",tString);
   
printf("%d\n",strlen(tString));

ConstructTree(test,tString,strlen(tString),&err);
printf("Type d'erreur: %d\n", err);
if (err==0) {
  printf("ComputeTree: %f\n",ComputeTree(test,1,1,&err));
  printf("Type d'erreur: %d\n", err);
  WriteTree(test,tString2);
  printf("%s\n",tString);
  printf("%s\n",tString2);
}

return 0;
}
